Re: [PATCH] inotify: fix one-shot support

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

 



Hi Robert,
hi John,

On Wednesday 08 February 2006 17:16, Robert Love wrote:
> On Wed, 2006-02-08 at 08:52 +0100, Ingo Oeser wrote:
> > See, now you can just pass IN_ONESHOT behavior flag without any
> > events to shoot at, which you couldn't do before. But this makes only
> > sense, if we would like to set a multi-shot mask to one-shot now.
> 
> Ack!

Ok, here comes the patch (against Linus' HEAD). 
It turned out, that we needed to change some more places to avoid having zero
events to watch for. If you are ok with it, I'll send it straight to Linus with
your Ack included and in proper patch format.


Regards

Ingo Oeser


diff --git a/fs/inotify.c b/fs/inotify.c
index 3041503..16ec5fb 100644
--- a/fs/inotify.c
+++ b/fs/inotify.c
@@ -935,6 +935,7 @@ asmlinkage long sys_inotify_add_watch(in
 	struct file *filp;
 	int ret, fput_needed;
 	int mask_add = 0;
+	int no_events = 0;
 	unsigned flags = 0;
 
 	filp = fget_light(fd, &fput_needed);
@@ -966,9 +967,13 @@ asmlinkage long sys_inotify_add_watch(in
 	if (mask & IN_MASK_ADD)
 		mask_add = 1;
 
-	/* don't let user-space set invalid bits: we don't want flags set */
+	/* Do we change and events or only multishot/singleshot? */
+	if (!(mask & IN_ALL_EVENTS))
+		no_events = 1;
+
+	/* Don't let user-space set invalid bits: we don't want flags set. */
 	mask &= IN_ALL_EVENTS | IN_ONESHOT;
-	if (unlikely(!mask)) {
+	if (unlikely(no_events && !mask_add)) {
 		ret = -EINVAL;
 		goto out;
 	}
@@ -987,6 +992,15 @@ asmlinkage long sys_inotify_add_watch(in
 		goto out;
 	}
 
+	/* 
+	 * Want to change only multishot/singleshot, 
+	 * but has no existing watch? -> Illegal -ioe 
+	 */
+	if (unlikely(no_events)) {
+		ret = -EINVAL;
+		goto out;
+	}
+
 	watch = create_watch(dev, mask, inode);
 	if (unlikely(IS_ERR(watch))) {
 		ret = PTR_ERR(watch);

-
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to [email protected]
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

[Index of Archives]     [Kernel Newbies]     [Netfilter]     [Bugtraq]     [Photo]     [Stuff]     [Gimp]     [Yosemite News]     [MIPS Linux]     [ARM Linux]     [Linux Security]     [Linux RAID]     [Video 4 Linux]     [Linux for the blind]     [Linux Resources]
  Powered by Linux